package de.duehl.vocabulary.japanese.updater.website.update.program;

import java.awt.Component;
import java.awt.Image;
import java.awt.Point;

import de.duehl.basics.version.Version;
import de.duehl.swing.ui.GuiTools;

import static de.duehl.vocabulary.japanese.updater.website.download.WebsiteFileContentDownloader.*;
import static de.duehl.vocabulary.japanese.updater.website.update.UpdateTools.informUserAboutDownloadFailure;

/**
 * Diese Klasse kümmert sich um Updates des Programms (Trainers).
 *
 * Diese Klasse wird aus dem Programm aufgerufen, nicht aus dem Launcher und auch nicht zum Startup
 * des Trainers, da das bereits der Launcher erledigt.
 *
 * @version 1.01     2024-11-11
 * @author Christian Dühl
 */

public class ProgramUpdater {

    /**
     * Die Version des laufenden Trainers, falls von diesem aufgerufen wird, oder des neusten
     * vorliegenden Trainers, falls vom Launcher aufgerufen wird.
     */
    private final String versionNumberRunning;

    /** Die aufrufende Gui als Komponente zur Positionierung von Dialogen. */
    private final Component guiComponent;

    /** Die Position des Rahmens der Oberfläche, vor der dieser Dialog erzeugt wird. */
    private final Point parentLocation;

    /** Das anzuzeigende ProgrammIcon. */
    private final Image programImage;

    /**
     * Konstruktor.
     *
     * @param versionNumberRunning
     *            Die Version des laufenden Trainers, falls von diesem aufgerufen wird, oder des
     *            neusten vorliegenden Trainers, falls vom Launcher aufgerufen wird.
     * @param guiComponent
     *            Die aufrufende Gui als Komponente zur Positionierung von Dialogen.
     * @param parentLocation
     *            Die Position des Rahmens der Oberfläche, vor der dieser Dialog erzeugt wird.
     * @param programImage
     *            Das anzuzeigende ProgrammIcon.
     */
    public ProgramUpdater(String versionNumberRunning,
            Component guiComponent, Point parentLocation, Image programImage) {
        this.versionNumberRunning = versionNumberRunning;
        this.guiComponent = guiComponent;
        this.parentLocation = parentLocation;
        this.programImage = programImage;
    }

    /** Prüft auf eine aktuellere Version des Programms. */
    public void update() {
        String versionNumberOnServer = ProgramUpdaterTools.downloadProgramVersionFromServer();
        if (versionNumberOnServer.equals(NO_DOWNLOAD_SUCCESS)) {
            informUserAboutDownloadFailure("des Programms", parentLocation, programImage);
        }
        else {
            Version versionOnServer = new Version(versionNumberOnServer, "interessiert nicht");
            Version versionRunning = new Version(versionNumberRunning, "interessiert nicht");
            if (versionOnServer.isNewerThan(versionRunning)) {
                informUserAboutNewerProgramVersion(versionNumberRunning, versionNumberOnServer);
            }
            else {
                informUserAboutProgramVersionIsActual(versionNumberRunning);
            }
        }
    }

    private void informUserAboutNewerProgramVersion(String verionNumberRunning,
            String versionNumberOnServer) {
        String title = "Es gibt eine neuere Programmversion";
        String message = "Es gibt eine neuere Programmversion!\n\n"
                + "Aktuelle Programmversion: " + verionNumberRunning + "\n"
                + "Neuere Programmversion: " + versionNumberOnServer;
        /*
        String beforeUrl = "Bitte besuchen sie die Seite";
        String url = SHORT_VOCBULARY_TRAINER_WEBPAGE_URL;
        String afterUrl = "und laden sie die neue Version herunter.";
        NewUpdateFoundDialog dialog = new NewUpdateFoundDialog(title, message, beforeUrl, url,
                afterUrl, parentLocation, programImage);
        dialog.setVisible(true);
        */
        message += "\n\n"
                + "Bitte beenden sie den Trainer und starten Sie den Launcher, um die neue Version "
                + "herunterzuladen.";
        GuiTools.informUser(title, message);
    }

    private void informUserAboutProgramVersionIsActual(String verionNumberRunning) {
        String title = "Der Vokabeltrainer ist aktuell";
        String message = "Die verwendete Programmversion " + verionNumberRunning + " ist aktuell.";
        GuiTools.informUserInEdt(guiComponent, title, message);
    }

}
